typedef struct SparRow *csptr;
typedef struct SparRow {
/*--------------------------------------------- 
| C-style CSR format - used internally
| for all matrices in CSR format 
|---------------------------------------------*/
   int n;
  /*  length of each row */
   int *nnzrow;
  /* pointer-to-pointer to store nonzero entries */
   double **ma;
  /* pointer-to-pointer to store column indices  */
   int **ja;
} SparMat;
/* -------------------------------------------------------------------*/
typedef struct PerMat4 *p4ptr;
typedef struct PerMat4 {
/*------------------------------------------------------------
| struct for storing the block LU factorization 
| contains all the block factors except the 
| data related to the last block. 
| n       = size of current block
| symperm = whether or not permutations are symmetric.
|           used only in cleanP4..
| nB      = size of B-block
| L, U    = ILU factors of B-block
| F, E    = sparse matrices in (1,2) and (2,1) 
|           parts of matrix. 
| perm    = (symmetric) permutation used in factorization
|           comes from the independent set ordering
| rperm   = unsymmetric permutation (rows) not used in this
|           version -- but left here for compatibility..
| D1, D2  = diagonal matrices (left, right) used for scaling
|           if scaling option is turned on. Note that the 
|           method works by scaling the whole matrix first
|           (at any level) before anything else is done. 
| wk     = a work vector of length n needed for various tasks
|            [reduces number of calls to malloc]           
|----------------------------------------------------------*/ 
  int n;                  
  int nB; 
  int symperm;
  /*   LU factors  */
  struct SparRow *L;
  struct SparRow *U;
  /* E, F blocks   */
  struct SparRow *E;
  struct SparRow *F;
  int *rperm;       /* row permutation    */ 
  int *perm;        /* col. permutation   */
  double *D1 ;
  double *D2 ;
  double *wk;
  /* pointer to next and previous struct */
  p4ptr prev; 
  p4ptr next;
} Per4Mat; 
/* -------------------------------------------------------------------*/
typedef struct ILUTfac *ilutptr;
typedef struct ILUTfac {
/*------------------------------------------------------------
| struct for storing data related to the last schur complement 
| we need to store the C matrix associated with the last block
| and the ILUT factorization of the related Schur complement.
| 
| n       = size of C block = size of Schur complement
| C       = C block of last level matrix. 
| L, U    = ILU factors of last schur complement. 
|
| meth[4] = parameters for defining variants in factorization 
|           - see function readin for details
| rperm    = row permutation used for very nonsymmetric matrices 
|            [such as bottleneck transversal] -- NOT IN THIS VERSION
| perm2     = unsymmetric permutation (columns) - used primarily
|           for the ILUTP version of ILUT/.. 
| D1, D2  = diagonal matrices (left, right) used for scaling
|           if scaling option is turned on. Note that the 
|           method works by scaling the whole matrix first
|           (at any level) before anything else is done. 
| wk     = a work vector of length n needed for various tasks
|            [reduces number of calls to malloc]           
|-----------------------------------------------------------*/
   int n;                  
  /*-------------------- C matrix of last block */
   struct SparRow *C;
  /* LU factorization       */
   struct SparRow *L;
   struct SparRow *U;
  /*--------------------  related to variants and methods */
  /*    int meth[4];   */
  int *rperm;   /* row single-sinded permutation */
  int *perm;    /* column perm .                */
  int *perm2;   /* column permutation coming from pivoting in ILU */ 
   double *D1;
   double *D2;
   double *wk;
} IluSpar;

typedef struct arms_st *arms;
typedef struct arms_st {
  /* this is the arms preconditioner struct 
  | it consists of a linked list of p4mat structs
  | and the ILUt factorization (inthe form of an 
  | IluSpar struct  
  |---------------------------------------------- */
  int nlev;
  ilutptr ilus;
  p4ptr levmat;
} armsMat;

typedef int idxtype;

typedef struct Graph_t *GraphPtr; 
typedef struct Graph_t {
  /* simplied version of metis graph == will be updated later to match metis
     definition so as to use nested dissections */
  int nvtxs, nedges;	  /* The # of vertices and edges in the graph */
  idxtype *xadj;	  /* Pointers to the locally stored vertices */
  idxtype *adj;	          /* Array storing the adjacency lists */
  idxtype *label;
} GType; 
/*-------------------- old-style csr matrix struct                      */
typedef struct CSRregular *csrCmat;
typedef struct CSRregular{
double *a;
int *ja;
int *ia;
} CSRMat;






